#include "thumbnailcache.h"

#include <QCoreApplication>
#include <qcoreevent.h>

#if (QT_VERSION < QT_VERSION_CHECK(4, 7, 0))
uint qHash(const QUrl &url) { return qHash(url.toString()); }
#endif

class Thumbnail
{
public:
    Thumbnail(const QUrl &url)
        : url(url)
    {
    }

    const QUrl url;
    QImage image;
};

class ThumbnailEvent : public QEvent
{
public:
    ThumbnailEvent(const QUrl &url, const QImage &image)
        : QEvent(QEvent::User)
        , url(url)
        , image(image)
    {
    }

    const QUrl url;
    const QImage image;
};

ThumbnailCache::ThumbnailCache(QCache<QUrl, Thumbnail> *cache, QObject *parent) :
    QObject(parent),
    m_cache(cache)
{
}

ThumbnailCache::~ThumbnailCache()
{
}

QImage ThumbnailCache::thumbnail(const QUrl &url, const QSize &scale)
{
    if (Thumbnail *thumbnail = m_cache->object(url)) {
        return (!scale.isEmpty()) && (!thumbnail->image.isNull())
                ? thumbnail->image.scaled(scale, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)
                : thumbnail->image;
    }
    else {
        m_urls.enqueue(url);
        m_cache->insert(url, new Thumbnail(url));

        if (m_urls.size() == 1) {
            emit downloadThumbnail(url);
        }

        return QImage();
    }
}

void ThumbnailCache::nextThumbnail() {
    if (!m_urls.isEmpty()) {
        emit downloadThumbnail(m_urls.dequeue());
    }
}

bool ThumbnailCache::event(QEvent *event)
{
    if (event->type() == QEvent::User) {
        ThumbnailEvent *thumbnailEvent = static_cast<ThumbnailEvent *>(event);

        if (Thumbnail *thumbnail = m_cache->object(thumbnailEvent->url)) {
            thumbnail->image = thumbnailEvent->image;

            emit thumbnailReady();

            nextThumbnail();
        }

        return true;
    }
    else {
        return QObject::event(event);
    }
}

void ThumbnailCache::onThumbnailDownloaded(QNetworkReply *reply) {
    QUrl url = reply->request().url();
    QImage image;
    image.loadFromData(reply->readAll());
    reply->deleteLater();
    QCoreApplication::postEvent(this, new ThumbnailEvent(url, image));
}
